#include <stdlib.h>
#include <unistd.h>
#include <ctpublic.h>
#include <ctlib.h>

static char  software_version[]   = "$Id: ct.c,v 1.9 1999/01/11 05:12:37 camber Exp $";
static void *no_unused_var_warn[] = {software_version,
                                     no_unused_var_warn};


int (*g_ctlib_msg_handler)() = NULL;
int (*g_ctlib_err_handler)() = NULL;
extern int (*g_tds_msg_handler)();
extern int (*g_tds_err_handler)();

static void _ct_bind_data(CS_COMMAND *cmd);


CS_RETCODE ct_exit(CS_CONTEXT *ctx, CS_INT unused)
{
	return CS_SUCCEED;
}
CS_RETCODE ct_init(CS_CONTEXT *ctx, CS_INT version)
{
   /* set the functions in the TDS layer to point to the correct info/err
    * message handler functions */
   g_tds_msg_handler = ctlib_handle_info_message;
   g_tds_err_handler = ctlib_handle_err_message;
	return CS_SUCCEED;
}
CS_RETCODE ct_con_alloc(CS_CONTEXT *ctx, CS_CONNECTION **con)
{
	*con = (CS_CONNECTION *) malloc(sizeof(CS_CONNECTION));
	(*con)->tds_login = (void *) tds_alloc_login();

	/* so we know who we belong to */
	(*con)->ctx = ctx;

	/* set default values */
	tds_set_library((*con)->tds_login, "CT-Library");
	tds_set_charset((*con)->tds_login, "iso_1");
	tds_set_packet((*con)->tds_login, 512);
	return CS_SUCCEED;
}
CS_RETCODE ct_callback()
{
	return CS_SUCCEED;
}
CS_RETCODE ct_con_props(CS_CONNECTION *con, CS_INT action, CS_INT property,
CS_VOID *buffer, CS_INT buflen, CS_INT *out_len)
{
	/* FIX ME, this will break if buflen != CS_NULLTERM */
	if (action==CS_SET) {
		switch (property) {
			case CS_USERNAME:
				tds_set_user(con->tds_login,(char *)buffer);
				break;
			case CS_PASSWORD:
				tds_set_passwd(con->tds_login,(char *)buffer);
				break;
			default:
				fprintf(stderr,"Unknown property %d\n",property);
		}
	} /* else if */
	return CS_SUCCEED;
}
CS_RETCODE ct_connect(CS_CONNECTION *con, CS_CHAR *servername, CS_INT snamelen)
{
char *server;
int needfree=0;

	if (snamelen==0 || snamelen==CS_UNUSED) {
		server=NULL;
	} else if (snamelen==CS_NULLTERM) {
		server=(char *)servername;
	} else {
		server = (char *) malloc(snamelen+1);
		needfree++;
		strncpy(server,servername,snamelen);
		server[snamelen]='\0';
	}
        tds_set_server(con->tds_login,server);
        con->tds_socket = (void *) tds_connect(con->tds_login);

        tds_set_parent( con->tds_socket, con);

	if (needfree) free(server);

	return CS_SUCCEED;
}
CS_RETCODE ct_cmd_alloc(CS_CONNECTION *con, CS_COMMAND **cmd)
{

	*cmd = (CS_COMMAND *) malloc(sizeof(CS_COMMAND));
	memset(*cmd,'\0',sizeof(CS_COMMAND));

	/* so we know who we belong to */
	(*cmd)->con = con;

	return CS_SUCCEED;
}
CS_RETCODE ct_command(CS_COMMAND *cmd, CS_INT type, CS_VOID *buffer, CS_INT buflen, CS_INT option)
{
int query_len;

	/* FIX ME -- will only work for type CS_LANG_CMD */
	if (buflen==CS_NULLTERM) {
		query_len = strlen(buffer);
	} else {
		query_len = buflen;
	}
	if (cmd->query) free(cmd->query);
	cmd->query = (char *) malloc(query_len + 1);
	strncpy(cmd->query,(char *)buffer,query_len);
	cmd->query[query_len]='\0';

	return CS_SUCCEED;
}
CS_RETCODE ct_send(CS_COMMAND *cmd)
{
	tds_submit_query(cmd->con->tds_socket, cmd->query);
	return CS_SUCCEED;
}
CS_RETCODE ct_results(CS_COMMAND *cmd, CS_INT *result_type)
{
	if (cmd->cmd_done) {
		cmd->cmd_done = 0;
		*result_type = CS_CMD_DONE;
		return CS_SUCCEED;
	} 
	switch (tds_process_result_tokens(cmd->con->tds_socket)) {
		case TDS_SUCCEED:
			if (cmd->con->tds_socket->res_info->rows_exist) {
				*result_type = CS_ROW_RESULT;
				cmd->cmd_done = 1;
			} else {
				*result_type = CS_CMD_SUCCEED;
			}
			return CS_SUCCEED;
		case TDS_NO_MORE_RESULTS:
			*result_type = CS_CMD_DONE;
			return CS_END_RESULTS;
		default:
			return CS_FAIL;
	}	
	return CS_FAIL;
}
CS_RETCODE ct_bind(CS_COMMAND *cmd, CS_INT item, CS_DATAFMT *datafmt, CS_VOID *buffer, CS_INT *copied, CS_SMALLINT *indicator)
{
TDSCOLINFO * colinfo;
TDSRESULTINFO * resinfo;
TDSSOCKET * tds;

   tds = (TDSSOCKET *) cmd->con->tds_socket;
   resinfo = tds->res_info;
   colinfo = resinfo->columns[item-1];
   colinfo->varaddr = (char *)buffer;
   colinfo->column_bindtype = datafmt->datatype;
   colinfo->column_bindlen = datafmt->maxlength;

	return CS_SUCCEED;
}
CS_RETCODE ct_fetch(CS_COMMAND *cmd, CS_INT type, CS_INT offset, CS_INT option, CS_INT *rows_read)
{
	switch (tds_process_row_tokens(cmd->con->tds_socket)) {
		case TDS_SUCCEED:
			*rows_read = 1;
			_ct_bind_data(cmd);
			return TDS_SUCCEED;
		case TDS_NO_MORE_ROWS:
			*rows_read = 1;
			_ct_bind_data(cmd);
			return CS_END_DATA;
		default:
			*rows_read = 0;
			return CS_FAIL;
	}
	return CS_SUCCEED;
}
static void _ct_bind_data(CS_COMMAND *cmd)
{
int i;
TDSCOLINFO *curcol;
TDSRESULTINFO *resinfo = cmd->con->tds_socket->res_info;
unsigned char *src;
int srctype,desttype;

   for (i=0;i<resinfo->num_cols;i++)
   {
      curcol = resinfo->columns[i];
	/* currently there is no NULL handling for ctlib, this is where it 
	** will be */
	/* printf("%d %s\n",i,resinfo->columns[i]->column_value); */
		if (curcol->varaddr) {
			/* FIX ME -- I only work with strings so far */
			/* strcpy(curcol->varaddr,  */
			src = &(resinfo->current_row[curcol->column_offset]);
			srctype = curcol->column_type;
			desttype = _ct_get_server_type(curcol->column_bindtype);
			tds_convert(srctype,src, -1,
				desttype,curcol->varaddr,curcol->column_bindlen);
		}
	}
}
CS_RETCODE ct_cmd_drop(CS_COMMAND *cmd)
{
	if (cmd) free(cmd);
	return CS_SUCCEED;
}
CS_RETCODE ct_close(CS_CONNECTION *con, CS_INT option)
{
	close(con->tds_socket->s);
	return CS_SUCCEED;
}
CS_RETCODE ct_con_drop(CS_CONNECTION *con)
{
	if (con) free(con);
	return CS_SUCCEED;
}
int _ct_get_client_type(int datatype)
{
   switch (datatype) {
         case SYBCHAR:
         case SYBVARCHAR:
		return CS_CHAR_TYPE;
		break;
         case SYBINT4:
      		return CS_INT_TYPE;
		break;
         case SYBINT2:
      		return CS_SMALLINT_TYPE;
		break;
         case SYBINT1:
      		return CS_TINYINT_TYPE;
		break;
         case SYBREAL:
      		return CS_REAL_TYPE;
		break;
         case SYBFLT8:
      		return CS_FLOAT_TYPE;
		break;
         case SYBMONEY:
      		return CS_MONEY_TYPE;
		break;
         case SYBMONEY4:
      		return CS_MONEY4_TYPE;
		break;
         case SYBDATETIME:
      		return CS_DATETIME_TYPE;
		break;
         case SYBDATETIME4:
      		return CS_DATETIME4_TYPE;
		break;
   }
}
int _ct_get_server_type(int datatype)
{
   switch (datatype) {
      case CS_CHAR_TYPE:
         return SYBCHAR;
         break;
      case CS_INT_TYPE:
         return SYBINT4;
         break;
      case CS_SMALLINT_TYPE:
         return SYBINT2;
         break;
      case CS_TINYINT_TYPE:
         return SYBINT1;
         break;
      case CS_REAL_TYPE:
         return SYBREAL;
         break;
      case CS_FLOAT_TYPE:
         return SYBFLT8;
         break;
      case CS_MONEY_TYPE:
         return SYBMONEY;
         break;
      case CS_MONEY4_TYPE:
         return SYBMONEY4;
         break;
      case CS_DATETIME_TYPE:
         return SYBDATETIME;
         break;
      case CS_DATETIME4_TYPE:
         return SYBDATETIME4;
         break;
      default:
         return -1;
         break;
   }
}
CS_RETCODE ct_cancel(CS_CONNECTION *conn, CS_COMMAND *cmd, CS_INT type)
{
}
CS_RETCODE ct_describe(CS_COMMAND *cmd, CS_INT item, CS_DATAFMT *datafmt)
{
TDSRESULTINFO *resinfo = cmd->con->tds_socket->res_info;
TDSCOLINFO *curcol;

	if (item<1 || item>resinfo->num_cols) return CS_FAIL;	
	curcol=resinfo->columns[item-1];
	strncpy(datafmt->name, curcol->column_name, CS_MAX_NAME);
	datafmt->namelen = strlen(curcol->column_name);
	/* need to turn the SYBxxx into a CS_xxx_TYPE */
	datafmt->datatype = _ct_get_client_type(curcol->column_type); 
	datafmt->maxlength = curcol->column_size;
	datafmt->usertype = curcol->column_usertype; 
	datafmt->precision = curcol->column_prec; 
	datafmt->scale = curcol->column_scale; 
	/* we currently don't store this info.  default it for now */
	datafmt->status = 0;
	datafmt->count = 1;
	datafmt->locale = NULL;
	
	return CS_SUCCEED;
}
CS_RETCODE ct_res_info(CS_COMMAND *cmd, CS_INT type, CS_VOID *buffer, CS_INT buflen, CS_INT *out_len)
{
TDSRESULTINFO *resinfo = cmd->con->tds_socket->res_info;

	switch(type) {
		case CS_NUMDATA:
			memcpy(buffer, &(resinfo->num_cols), sizeof(int));
			break;
		case CS_ROW_COUNT:
			memcpy(buffer, &(resinfo->row_count), sizeof(long));
			break;
		default:
			fprintf(stderr,"Unknown type in ct_res_info: %d\n",type);
			return CS_FAIL;
			break;
	}
	return CS_SUCCEED;
}
CS_RETCODE ct_config(CS_CONTEXT *ctx, CS_INT action, CS_INT property, CS_VOID *buffer, CS_INT buflen, CS_INT *outlen)
{
	return CS_SUCCEED;
}
CS_RETCODE ct_cmd_props(CS_COMMAND *cmd, CS_INT action, CS_INT property, CS_VOID *buffer, CS_INT buflen, CS_INT *outlen)
{
	return CS_SUCCEED;
}
CS_RETCODE ct_compute_info(CS_COMMAND *cmd, CS_INT type, CS_INT colnum, CS_VOID *buffer, CS_INT buflen, CS_INT *outlen)
{
	return CS_SUCCEED;
}
CS_RETCODE ct_get_data(CS_COMMAND *cmd, CS_INT item, CS_VOID *buffer, CS_INT buflen, CS_INT *outlen)
{
	return CS_SUCCEED;
}
CS_RETCODE ct_capability(CS_CONNECTION *con, CS_INT action, CS_INT type, CS_INT capability, CS_VOID *value)
{
	return CS_SUCCEED;
}
CS_RETCODE ct_dynamic(CS_COMMAND *cmd, CS_INT type, CS_CHAR *id, CS_INT idlen, CS_CHAR *buffer, CS_INT buflen)
{
	return CS_SUCCEED;
}
CS_RETCODE ct_param(CS_COMMAND *cmd, CS_DATAFMT *datafmt, CS_VOID *data, CS_INT datalen, CS_SMALLINT indicator)
{
}
