#include <sql.h>
#include <tds.h>

static char  software_version[]   = "$Id: odbc.c,v 1.6 1999/01/11 05:12:38 camber Exp $";
static void *no_unused_var_warn[] = {software_version,
                                     no_unused_var_warn};


RETCODE SQLGetTypeInfo(SQLHSTMT hstmt, SQLSMALLINT sql_type)
{
	return SQL_SUCCESS;
}
RETCODE SQLAllocEnv(SQLHENV *penv)
{
        *penv = (SQLHENV) malloc(sizeof(struct _henv));
        memset(*penv,'\0',sizeof(struct _henv));
	return SQL_SUCCESS;
}
RETCODE SQLAllocConnect(SQLHENV henv, SQLHDBC *phdbc)
{
        *phdbc = (SQLHDBC) malloc(sizeof(struct _hdbc));
        memset(*phdbc,'\0',sizeof(struct _hdbc));
	(*phdbc)->henv=henv;
	(*phdbc)->tds_login= (void *) tds_alloc_login();

	return SQL_SUCCESS;
}
RETCODE SQLConnect(SQLHDBC hdbc, SQLCHAR *svr_name, SQLSMALLINT svr_name_len, 
	SQLCHAR *user_name, SQLSMALLINT user_name_len,
	SQLCHAR *password, SQLSMALLINT password_len)
{
        tds_set_server(hdbc->tds_login,svr_name);
        tds_set_user(hdbc->tds_login,user_name);
        tds_set_passwd(hdbc->tds_login,password);
        hdbc->tds_socket = (void *) tds_connect(hdbc->tds_login);

	return SQL_SUCCESS;
}
RETCODE SQLError(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt, 
	SQLCHAR *szSqlState, SQLINTEGER *pfNativeError, SQLCHAR *szErrorMsg, 
	SQLSMALLINT cbErrorMsgMax, SQLSMALLINT *pcbErrorMsg)
{
	return SQL_SUCCESS;
}
RETCODE SQLSetConnectOption(SQLHDBC hdbc, int option, int value)
{
	return SQL_SUCCESS;
}
RETCODE SQLAllocStmt(SQLHDBC hdbc, SQLHSTMT *phstmt)
{
        *phstmt = (SQLHSTMT) malloc(sizeof(struct _hstmt));
        memset(*phstmt,'\0',sizeof(struct _hstmt));
	(*phstmt)->hdbc=hdbc;
	return SQL_SUCCESS;
}
RETCODE SQLPrepare(SQLHSTMT hstmt, SQLCHAR *stmt_text, SQLINTEGER test_len)
{
	if (test_len!=SQL_NTS) return SQL_FAILURE;
	strcpy(hstmt->query, stmt_text);
	return SQL_SUCCESS;
}
RETCODE SQLBindParameter(SQLHSTMT hstmt, SQLUSMALLINT param_num, 
	SQLSMALLINT in_out_type, SQLSMALLINT value_type, 
	SQLSMALLINT param_type, SQLUINTEGER column_size,
	SQLSMALLINT decimal_digits, SQLPOINTER pparam_value,
	SQLINTEGER buffer_len, SQLINTEGER *pstr_len) 
{
TDSCOLINFO * colinfo;
TDSRESULTINFO * resinfo;
TDSSOCKET * tds;
struct _sql_bind_info *bindinfo;

/* stash this in the _sql_bind_info linked list until we have a result set to 
 * use it with. SQLCloseCursor is responsible for cleaning this list up.
 */
	bindinfo = hstmt->bind_head;
	if (!bindinfo) {
		bindinfo = (struct _sql_bind_info *) malloc(sizeof(struct _sql_bind_info));
        	memset(bindinfo,'\0',sizeof(struct _sql_bind_info));
		hstmt->bind_head = bindinfo;
	} else {
		while (bindinfo->next) {
			bindinfo = bindinfo->next;
		}
		bindinfo->next  = (struct _sql_bind_info *) malloc(sizeof(struct _sql_bind_info));
		bindinfo = bindinfo->next;
        	memset(bindinfo,'\0',sizeof(struct _sql_bind_info));
	}
	bindinfo->column_number = param_num;
        bindinfo->varaddr = (char *)pparam_value;
        bindinfo->column_bindtype = value_type;
        bindinfo->column_bindlen = buffer_len;

	return SQL_SUCCESS;
}
RETCODE SQLExecute(SQLHSTMT hstmt)
{
	if (tds_submit_query(hstmt->hdbc->tds_socket, hstmt->query)==TDS_SUCCEED) {
		return SQL_SUCCESS;
	} else {
		return SQL_FAILURE;
	}
}
RETCODE SQLExecuteDirect(SQLHSTMT hstmt, SQLCHAR *stmt_text, SQLINTEGER test_len)
{
	strcpy(hstmt->query, stmt_text);
	if (tds_submit_query(hstmt->hdbc->tds_socket, hstmt->query)==TDS_SUCCEED) {
		return SQL_SUCCESS;
	} else {
		return SQL_FAILURE;
	}
}
RETCODE SQLFetch(SQLHSTMT hstmt)
{
static int state = 0;
int ret;
struct _sql_bind_info *bindinfo;
TDSSOCKET *tds;
TDSRESULTINFO * resinfo;
TDSCOLINFO *colinfo;
int i;
	
	/* does anyone know how ODBC deals with multiple result sets? */
	if (state==0) {
		ret = tds_process_result_tokens(hstmt->hdbc->tds_socket);
		if (ret==TDS_NO_MORE_RESULTS) {
			return SQL_NO_DATA;
		} else if (ret==TDS_SUCCEED) {
			state++;
			/* handle binding here */
			bindinfo = hstmt->bind_head;
			while (bindinfo) {
				tds = hstmt->hdbc->tds_socket;
				colinfo = tds->res_info->columns[bindinfo->column_number-1];
				colinfo->varaddr         = bindinfo->varaddr;
				colinfo->column_bindtype = bindinfo->column_bindtype;
				colinfo->column_bindlen  = bindinfo->column_bindlen;
				bindinfo = bindinfo->next;
			}
			return SQL_SUCCESS;
		} else {
			return SQL_FAILURE;
		}
	} else {
 		ret = tds_process_row_tokens(hstmt->hdbc->tds_socket);
		state = 0;
		/* FIX ME -- commonize type conversion, only dblib works */
		tds = hstmt->hdbc->tds_socket;
		resinfo = tds->res_info;
		for (i=0;i<resinfo->num_cols;i++) {
      			colinfo = resinfo->columns[i];
			if (colinfo->varaddr) {
				strcpy(colinfo->varaddr, 
				&resinfo->current_row[colinfo->column_offset]);
			}
		}
		if (ret!=TDS_NO_MORE_ROWS && ret!=TDS_SUCCEED) return SQL_FAILURE;
		else return SQL_SUCCESS;
	}
}
RETCODE SQLGetData(SQLHSTMT hstmt, UWORD column, SWORD type, PTR buffer, SDWORD buffer_len, SDWORD *data_len)
{
}
RETCODE SQLCloseCursor(SQLHSTMT hstmt)
{
struct _sql_bind_info *bindinfo,*prev;

	bindinfo = hstmt->bind_head;
	while (bindinfo) {
		prev = bindinfo;
		bindinfo = bindinfo->next;
		free(prev);
	}
	return SQL_SUCCESS;
}
